@@ -1,6 +1,6 @@
 /* lsh.h
  *
- * $Id: lsh.h,v 1.12 2000/02/05 17:13:12 nisse Exp $ */
+ * $Id: lsh.h,v 1.13 2000/02/22 20:48:19 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -122,4 +122,7 @@ struct randomness;
 struct sexp;
 struct sexp_iterator;
 
+/* userauth.h */
+struct user;
+
 #endif /* LSH_H_INCLUDED */
