@@ -5,7 +5,7 @@
  * extract the public key, encrypt the private key, and save the
  * results in two separate files.
  *
- * $Id: lsh_keygen.c,v 1.18 1999/11/07 21:02:09 nisse Exp $ */
+ * $Id: lsh_keygen.c,v 1.19 2000/02/22 20:50:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,6 +34,7 @@
 #include "publickey_crypto.h"
 #include "randomness.h"
 #include "sexp.h"
+#include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -50,6 +51,11 @@
 
 /* Option parsing */
 
+const char *argp_program_version
+= "lsh_keygen-" VERSION;
+
+const char *argp_program_bug_address = BUG_ADDRESS;
+
 /* GABA:
    (class
      (name lsh_keygen_options)
