@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.6 2000/03/14 21:05:01 nisse Exp $
+ * $Id: server_session.h,v 1.7 2000/03/26 21:20:27 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -35,7 +35,7 @@
 
 struct ssh_channel *
 make_server_session(struct user *user,
-		    UINT32 max_window,
+		    UINT32 initial_window,
 		    struct alist *request_types);
 struct command *
 make_server_connection_service(struct alist *session_requests);
