@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.5 2000/03/29 18:45:08 nisse Exp $
+ * $Id: unix_user.c,v 1.6 2000/04/02 17:34:19 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -279,7 +279,7 @@ do_exec_shell(struct user *u, int login,
   if (tz)
     envp[i++] = format_env_pair_c("TZ", tz);
 
-  assert(i < MAX_ENV);
+  assert(i <= MAX_ENV);
 #undef MAX_ENV
 
   for (j = 0; j<env_length; j++)
